package xyz.thastertyn.Scrape;

import java.io.IOException;
import java.util.ArrayList;
import java.util.regex.Pattern;

import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import xyz.thastertyn.Types.Choice;
import xyz.thastertyn.Types.FinalMark;
import xyz.thastertyn.Types.Option;
import xyz.thastertyn.Types.Options;
import xyz.thastertyn.Types.Predmet;
import xyz.thastertyn.Types.Znamka;

public class Znamky extends JecnaScrape {

	// schoolYear, schoolYearId
	private ArrayList<Predmet> predmety;

	// int znaci id roku, boolean jestli je jen prvni nebo i druhe pololeti
	private Options schoolYearOptions;
	private Options schoolHalfYearOptions;

	public void download() throws IOException
	{
		download("https://www.spsejecna.cz/score/student");
	}
	
	@Override
	public void download(Choice choice) throws IOException
	{
		download(String.format(
			"https://www.spsejecna.cz/score/student?schoolYearId=%s&schoolYearHalfId=%s", 
			choice.getChoices().get(0), 
			choice.getChoices().get(1)));
	}

	private void download(String url) throws IOException
	{
		schoolHalfYearOptions = new Options("Pololeti");
		schoolYearOptions = new Options("Skolni R.");
		
		predmety = new ArrayList<>();
		Document znamkyDokumentHTML = Downloader.download(url).get();

		// Predmety ulozene v <tr>
		Elements[] radkyPredmetuHTML = znamkyDokumentHTML
			.select("table.score")
			.select("tr")
			.stream()
			.map(Element::children)
			.toArray(Elements[]::new);

		int subjectIndex = 0;

		for(int i = 1; i < radkyPredmetuHTML.length; i++)
		{
			String plnyNazevPredmetu = radkyPredmetuHTML[i].get(0).text();

			String jmenoPredmetu = Pattern
				.compile("\\((.*?)\\)")
				.matcher(plnyNazevPredmetu)
				.results()
				.findFirst()
				.map(m -> m.group(1))
				.orElse(plnyNazevPredmetu);

			if(radkyPredmetuHTML[i].get(2).childrenSize() == 0) // Predmet jeste nema vyslednou znamku
			{
				predmety.add(new Predmet(jmenoPredmetu));
			}else{
				String vyslednaZnamka = radkyPredmetuHTML[i].get(2).select("a.scoreFinal").text();
				predmety.add(new Predmet(jmenoPredmetu, FinalMark.fromValue(vyslednaZnamka)));
			}

			for(Element znamkaElement : radkyPredmetuHTML[i].get(1).select("a.score"))
			{
				int znamka;
				boolean jeMala = false;

				String textZnamky = znamkaElement.select("span.value").text();

				znamka = textZnamky.matches("\\d") ?
					Integer.parseInt(textZnamky)
					:
					-1; // Nejspis se jedna o N (Nehodnocen)

				// Mala znamka se bude pocitat jako polovicni vaha
				jeMala = znamkaElement.hasClass("scoreSmall");

				predmety.get(subjectIndex).addZnamka(new Znamka(znamka, jeMala, textZnamky));
			}

			predmety.get(subjectIndex).calculateFinalMark();

			subjectIndex++;
		}

		Element optionsPanel = znamkyDokumentHTML.selectFirst("form.listConfigure");
		
		Elements skolniRoky = optionsPanel.select("select[id=schoolYearId]").select("option");
		Elements pololeti = optionsPanel.select("select[id=schoolYearHalfId]").select("option");

		
		for(Element e : skolniRoky)
		{
			boolean isDefault = e.hasAttr("selected");
			schoolYearOptions.addOption(new Option(e.text(), e.attr("value"), isDefault));
		}

		for(Element e : pololeti)
		{
			boolean isDefault = e.hasAttr("selected");
			schoolHalfYearOptions.addOption(new Option(e.text(), e.attr("value"), isDefault));
		}
	}
	
	public ArrayList<Predmet> getPredmety()
	{
		return predmety;
	}

	public Options[] getOptions()
	{
		return new Options[] {schoolYearOptions, schoolHalfYearOptions};
	}
}